function [ft, main_xf, ff, runtime_f, Jf] = autoprox_1d(f, var, xbounds, ns, xinit, continuity)
% PWA approximation of a function of one variable

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

% checking the existence of the required features
[ret_value_fm,ret_value_s]=autoprox_have_fmincon_and_sym;

error(nargchk(3, 5, nargin));

opt = optimset('display', 'iter', ...  % 'off' | 'iter'
    'Algorithm','active-set', ...  % 'active-set' | 'interior-point'
    'GradObj', 'off', ...  % use 'on' to enable gradients
    'TolFun', 1e-6, ...
    'MaxFunEvals', 1e5, ...
    'MaxIter', 1e4, ...
    'LargeScale', 'off', ...
    'Simplex', 'on');

% input validation
numeric_integration = true;
if nargin < 6
    continuity = true;
end
if nargin < 5
    xinit = 'greedy';
end
if isa(f, 'sym')
    f = matlabFunction(f);
end
if isa(xinit, 'char') && isequal(xinit, 'greedy')
    xinit = autoprox_greedy(f, xbounds, ns, false); 
end

if isempty(xinit)
    x0 = zeros((ns-1)+2*ns, 1);  % [r, a, b]
elseif any(isnan(xinit(1:ns-1))) || any(isinf(xinit(1:ns-1)))
    warning('Unsuitable starting points, using empty instead.');
    x0 = zeros((ns-1)+2*ns, 1);  % [r, a, b];
elseif numel(xinit) == ns
    % convert lienarization points to regions
    x0 = points2regions(xinit, f); x0 = [x0(1:ns-1, 1); x0(:, 2); x0(:, 3)];
else
    x0 = [xinit(1:ns-1, 1); xinit(:, 2); xinit(:, 3)];
end

syms x a b
d = (f(x) - (a*x+b));

if numeric_integration
    % numeric integration
    J = (d)^2;
    Jf = matlabFunction(J, 'vars', {x, a, b});
    grad_f = {};
else
    % symbolic integration
    syms xl xu
    
    fprintf('Deriving symbolic representation of the integral...\n');
    tic
    J = int(d^2, xl, xu);
    toc
    grad = {}; grad_f = {};
    
    Jf = matlabFunction(J, 'vars', {a, b, xl, xu});
end

nx = length(x0);
z = zeros(1, nx);

%-------------------------------------------------
% inequality constraints
A = []; B = []; 

if ns > 1

    %  r(1) >= xmin
    q = z;
    q(1) = -1;
    A = [A; q];
    B = [B; -xbounds(1)];
    %  r(end) <= xmax
    q = z;
    q(ns-1) = 1;
    A = [A; q];
    B = [B; xbounds(2)];
    %  r(i) + tol <= r(i+1)
    tol = 1e-1;
    for i = 1:ns-2
        q = z;
        q(i) = 1; q(i+1) = -1;
        A = [A; q];
        B = [B; -tol];
    end
end


%-------------------------------------------------
% equality constraints
Aeq = []; Beq = [];

%-------------------------------------------------
% nonlinear constraints
% equality continuity constraints: a(i)*r(i)+b(i) == a(i+1)*r(i)+b(i+1)
if continuity
    constraints = @(x) autoprox_fmincon_constraints(x, ns);
else
    constraints = [];
end

%-------------------------------------------------
% bounds on optimization variables
Zmin = [xbounds(1)*ones(ns-1, 1); -1e6*ones(2*ns, 1)];
Zmax = [xbounds(2)*ones(ns-1, 1); 1e6*ones(2*ns, 1)];

%-------------------------------------------------
% solve the NLP problem
t = tic;
[xf, ff] = fmincon(@(x) autoprox_fmincon_cost(x, Jf, ns, xbounds, grad_f), ...
    x0, A, B, Aeq, Beq, ...
    Zmin, Zmax, ...
    constraints, ...
    opt);
runtime_f = toc(t);
xf = reshape([xf(1:ns-1); Inf; xf(ns:end)], ns, 3);
main_xf = cell(1,3);
main_xf{1} = xf;
main_xf{2} = var;
main_xf{3} = xbounds;


ft = multilin(xf);
